import java.util.Arrays;
/**
 * Die Klasse Sudoku stellt die Methoden zur Bearbeitung bereit
 * 
 * Die 0 kennzeichnet unbesetzte Felder
 * 
 * @author (claus albowski) 
 * @version (Juni 15)
 */
public class Sudoku
{
    private int[] sudoku;

    /**
     * Konstruktor für Objekte der Klasse Sudoku
     */
    public Sudoku(int[] sudoku)
    {
        this.sudoku = Arrays.copyOf(sudoku, 81);
    }
    
    /**
     * get-Methode
     */
    public int[] gibDaten() { return sudoku; }
    
    /**
     * zeigt die Belegung in der Konsole an
     */
    public void zeige()
    {
        System.out.print("|- - -|- - -|- - -|");
        for (int x=1; x<10;x++)
        {
            System.out.println(" ");
            for (int y=1; y<10;y++)
            System.out.print(" "+sudoku[bestimmeIndex(x,y)]);
        }
        System.out.println("\n|- - -|- - -|- - -|\n");
    }
    
    /**
     * prueft, ob die Position bereits belegt ist
     */
    public boolean istBelegt(int i)
    { return sudoku[i] != 0; }

    /**
     * belegt eine Position mit einem Wert nach Pruefung
     */
    public boolean belegeMit(int zeile, int spalte, int wert)
    {
        if (zeile<1) return false;
        if (spalte<1) return false;
        if (wert<1) return false;
        if (wert>9) return false;
        if (zeile>9) return false;
        if (spalte>9) return false;
        int i = bestimmeIndex(zeile,spalte);
        if (istBelegt(i)) return false;
        sudoku[i] = wert;
        if (istZulaessig(zeile,spalte)) return true;
        else
        {
            sudoku[i] = 0;
            return false;
        }
    }
    
    /**
     * berechnet den Index zu den zeile / spalte - Werten
     */
    private int bestimmeIndex(int zeile, int spalte)
    {
        return zeile*9+spalte - 10;
    }
    
    /**
     * prueft, ob die Belegung an der uebergebenen Position
     * zulaessig ist [unter der Voraussetzung, dass die anderen
     * Felder ok sind.]
     */
    public boolean istZulaessig(int zeile, int spalte)
    {
        int i = bestimmeIndex(zeile,spalte);
        return (zeileOK(i) && spalteOK(i) && blockOK(i));
    }
    
    /**
     * prueft die Zulaessigkeit der Zeile
     */
    public boolean zeileOK(int index)
    {
        int anfang = index - index%9;
        int anzahl;
        for (int j = 1; j<10; j++)
        {
            anzahl = 0;
            for (int i=0; i<9; i++)
            if (sudoku[anfang+i] == j) anzahl++;
            if (anzahl>1) return false;
        }
        return true;
    }
    
    /**
     * prueft die Zulaessigkeit der Spalte
     */
    public boolean spalteOK(int index)
    {
        int anfang = index%9;
        int anzahl;
        for (int j = 1; j<10; j++)
        {
            anzahl = 0;
            for (int i=0; i<9; i++)
            if (sudoku[anfang+i*9] == j) anzahl++;
            if (anzahl>1) return false;
        }
        return true;
    }
    
    /**
     * prueft die Zulaessigkeit des Blocks
     */
    public boolean blockOK(int index)
    {
        int anfangBlockZeile = 3*(index/27); // 0 basierend
        int anfangBlockSpalte = (index%9)/3; // 0 basierend
        int anfang = anfangBlockZeile*9 + anfangBlockSpalte*3;
        int anzahl;
        for (int j = 1; j<10; j++)
        {
            anzahl = 0;
            for (int i=0; i<3; i++)
            for (int k=0; k<3; k++)
            if (sudoku[anfang+i*9+k] == j) anzahl++;
            if (anzahl>1) return false;
        }
        return true;
    }
    
}
